pro mwa_prep, file, index, data, regrid=regrid, rotate=rotate, clip=clip, $
		plate_scale=plate_scale, casa_history=casa_history, runtime=runtime, $
		write=write, debug=debug, quiet=quiet, lstop=lstop, no_output=no_output, $
		iindex=iindex, idata=idata, nodata=nodata, dimensions=dimensions

; ========================================================================================
;+
; PROJECT: 
;	MWA
;
; NAME: 
;	MWA_PREP
;
; CATEGORY: 
;	FITS, Image alignment, File I/O
;
; PURPOSE: 
;	Read an MWA FITS image of the Sun and output an image and header structure that uses 
;	a standard solar coordinate convention (Helioprojective-Cartesian, TAN projection)  
;	with a header format that's compatible with various SolarSoft (SSWIDL) routines.  
;
; CALLING SEQUENCE:
;	mwa_prep, file, index, data [,regrid=boolean] [,rotate=boolean] [,clip=float] 
;		[,plate_scale=float] [,quiet=boolean] [,runtime=boolean] [,casa_history=string array] 
;		[,write=string] [,debug=boolean] [,lstop=boolean]
;
; INPUTS: 
;	FILE = String (array). FITS file path or array of file paths. Note that the input 
;		   can also be supplied using the IINDEX and IDATA keywords, in which case the 
;		   FILE input must still be supplied as an arbitrary string.
;
;	ALTERNATIVE KEYWORD INPUTS: 
;
;		IINDEX = Structure (array). Keyword input alternative to the FILE input. Useful if 
;			 	 you want to make some modification to the (meta)data before prepping. Note
;				 that the FILE input must still be supplied with the original filenames 
;				 so that certain information like the OBSID and DATE_OBS (for CASA files)
;				 can be extracted from them. 
;
;		IDATA = Float array [nX, nY, nIINDEX]. Keyword input for data corresponding to IINDEX.
;				Must be supplied with IINDEX.
;
; OUTPUTS: 
;	INDEX = Structure (array). FITS header structure adapted to Helioprojective-Cartesian
;			coordinates with the standard TAN projection. 	
;
;	DATA = Float Array [nX, nY, nImgs]. Output data array.
;      
; KEYWORDS: 
;	REGRID = Boolean, default=1. Disable this flag if you want to cut runtime by a factor 
;			 of 2 by skipping the proper re-gridding process. In principle, re-gridding is 
;			 necessary because the input MWA data uses SIN-projected celestial coordinates, 
;			 so the coordinate system can't just be converted to a TAN-projection system 
;			 without doing an interpolation onto the new grid. In practice, the difference 
;			 between doing this and not is usually only ~1-2 pixels (20-40 arcsec) within a 
;			 few solar radii of the Sun, so choosing not to re-grid won't hurt much and can
;			 save a lot of time if you're processing a lot of images. 
;
;	ROTATE = Boolean, default=1. Disable this flag if you don't want the image to be 
;			 rotated so that the solar N/S axis is up and the Sun is centered.
;
;	CLIP = Float, default=6 (only applicable if ROTATE=1). By default, the output image
;		   will be clipped so that the Sun is in the center and the total image dimensions 
;		   are CLIP solar radii in x and y. R_sun is taken to be 960" for this purpose, 
;		   so the default size is 5,760 arcsec. Set CLIP=0 to disable clipping. 
;
;	PLATE_SCALE = Float, default=20 (only applicable if ROTATE=1). By default, the output 
;				  image will be resized for a plate scale of 20 arcsec/pixel, which 
;				  corresponds roughly to the maximum spatial resolution (highest frequency)
;				  observations. This serves to produce uniform image dimensions and 
;				  spatial scales for all observations.
;	
;	CASA_HISTORY = String array. If the input FILE was created with CASA, then this variable 
;				   will contain the HISTORY information from the FITS header. CASA pours 
;				   a lot of output into HISTORY, such that the string arrays can have 
;				   >50,000 lines. This slows the header editing functions down significantly, 
;				   so the CASA history is just removed and put into this variable. Note that  
;				   CASA_HISTORY will contain only the history for the last element of the 
;				   FILE input. So if you want the history for each file, you need to run 
;				   MWA_PREP separately for each. 
;
;	WRITE = String or Boolean. Set to the directory you want prepped FITS files to be written 
;			to. Filenames will be the same as the input with '_prep' appended. WRITE may 
;			also simply be set to 1 and the FITS files will then be written to the same 
;			directory as the input FILE.  
;
;	NO_OUTPUT = Boolean. Set if you do NOT want the INDEX and DATA output arrays to be 
;				built. This is useful if you just want to write out prepped FITS files 
;				from an input FILE array that is arbitrarily long and may cause a memory 
;				overflow if all the images are stored in a single output array. 
;
;	QUIET = Boolean, set to disable messages.
;
;	RUNTIME = Boolean, set to print runtime at exit. 
;
;	DEBUG = Boolean, creates some extra variables for debugging / sanity checks. 
;
;	LSTOP = Boolean, debugging keyword to stop at the end of a single loop iteration. 
;
;	NODATA = Boolean, set to only read/output unmodified FITS header(s). 
;
;	DIMENSIONS = Integer Array [Nx, Ny]. Pixel dimensions of the output DATA array. 
;				 If PLATE_SCALE=20, ROTATE=1 and REGRID=1 (the default settings), then 
;				 DIMENSIONS defaults to [289,289]. Otherwise DIMENSIONS is undefined and 
;				 will be determined automatically based on the other keyword settings. 
;				 Defining DIMENSIONS ensures that images from different channels have the
;				 same dimensions, as occasionally images with different input plate scales 
;				 will vary in output dimensions by a single pixel after the rotation, 
;				 translation, and scaling operations (despite having accurate pointing 
;				 info in their FITS headers).
; 			
;
; EXAMPLES:
;		
;		file = '~./1130645040_187-188-t0009-XX-image.fits'
;		mwa_prep, file, index, data
;		wcs2map, data, fitshead2wcs(index), map
;		plot_map, map
;
; NOTES: (1) For CASA-generated images, the UTC time must be indicated in the filename 
;			 because the FITS headers will only contain start time of the overall OBSID. 
;			 The string tHHMMSS should be somewhere in the filename. The date will be 
;			 taken from the DATE_OBS field. This could be an issue if an OBSID happens 
;			 to straddle two days... 
;			
;		 (2) Will automatically load the SPICE ephemeris files, if not already loaded, via 
;		 	 CONVERT_STEREO_LONLAT. This will cause the first run in a given session to be 
;		 	 extra slow. 
;	
;		 (3) The routine will choke if you input a file list that includes both CASA and 
;			 WSClean FITS files because the different metadata can't be concatenated 
;			 smoothly. That will need to be fixed if we ultimately want a set of uniform 
;			 FITS headers for a self-consistent image archive. 
;
;		 (4) Files that were produced by MWA_PREP, as indicated by their FITS headers, 
;			 will just be read in using MREADFITS and output. So any routines that call 
;			 MWA_PREP don't need to be modified to handle prepped vs. un-prepped FITS. 
;
; CALLS: TRIM, MREADFITS, HEADFITS, FITSHEAD2WCS, WCS_GET_COORD, CONVERT_STEREO_LONLAT, 
;		 PB0R, MPFITELLIPSE, WCS_RECTIFY, WCS2MAP, SUB_MAP, GRID_MAP, MAP2WCS, 
;		 WCS2FITSHEAD, SXADDHIST, SXADDPAR, FITSHEAD2STRUCT, CONCAT_STRUCT, PROGRESS_BAR, 
;		 REM_TAG
;
;		 Requires a basic SSWIDL installation along with the STEREO/SSC branch. 
;
; MODIFICATION HISTORY: 
;	 - 2016/07/01 - Written by Patrick McCauley (mccauley.pi@gmail.com)
;	 - 2016/07/14 - Added handling for CASA-generated files, FITS writing - PIM
;	 - 2016/07/15 - Moved CASA time fix up in code for solar position calculation. Also 
;					added handling to check if FITS file was already prepped - PIM 
;	 - 2016/10/25 - Added NO_OUTPUT keyword, modified the WRITE keyword, and added 
;					added additional metadata / quality metrics to the header. - PIM
;	 - 2016/11/30 - Added IINDEX and IDATA keyword input options, and removed linefeed 
;					character from the ORIGIN header tag of CASA-generated files. - PIM
;	 - 2017/01/02 - Added NODATA keyword. - PIM
;	 - 2017/02/13 - Removed !CONST variable for compatibility with older IDL versions - PIM 
;	 - 2017/04/12 - Added DIMENSIONS keyword. - PIM
;	 - 2017/04/21 - Fixed bug in beam angle rotation from celestial -> solar coords -PIM
;-
; ========================================================================================

t1 = systime(/seconds)

;versions
progname = 'MWA_PREP.PRO'
progver = 'V0.0' ; 2016/07/01 (PIM)
progver = 'V0.1' ; 2016/07/14 (PIM)
progver = 'V0.2' ; 2016/07/15 (PIM)
progver = 'V0.3' ; 2016/10/25 (PIM)
progver = 'V0.4' ; 2016/11/30 (PIM)
progver = 'V0.5' ; 2017/01/02 (PIM)
progver = 'V0.6' ; 2017/02/13 (PIM)
progver = 'V0.7' ; 2017/04/12 (PIM)
progver = 'V0.8' ; 2017/04/21 (PIM)


;---Input type determination and error handling

if n_elements(iindex) gt 0 OR n_elements(idata) gt 0 then begin
	in_type = 'keyword'	
	
	nimgs = n_elements(iindex)
	sz = size(idata, /dimensions)
	
	if n_elements(file) ne nimgs then begin
		message, 'Oops, n_elements(FILE) must match n_elements(IINDEX).', /info
		return
	endif	
	
	if nimgs eq 0 OR n_elements(sz) lt 2 then begin
		message, 'Oops, INDEX and IDATA must be supplied together.', /info
		return
	endif	
	
	if n_elements(sz) eq 2 then ndata = 1 else ndata = sz[2]
	if ndata ne nimgs then begin
		message, 'Oops, IINDEX and IDATA dimensions do not match.', /info
		return
	endif

endif else begin
	in_type = 'default'
	
	if n_elements(file) eq 0 then begin
		message, 'Oops, missing FILE input.', /info
		return
	endif	
	
	nimgs = n_elements(file)
	
	if total(file_test(file)) ne nimgs then begin
		message, 'Oops, not all input files exist.', /info
		return
	endif
	
	if keyword_set(nodata) then begin 
		mreadfits, file, index, /nodata
		return
	endif
	
endelse

;---Keyword Defaults

if n_elements(rotate) eq 0 then rotate=1
if n_elements(regrid) eq 0 then regrid=1
if keyword_set(rotate) then begin
	if n_elements(clip) eq 0 then clip = 6.
	if n_elements(plate_scale) eq 0 then plate_scale = 20.
endif

if rotate eq 1 AND regrid eq 1 AND plate_scale eq 20 AND clip eq 6 then $
	if n_elements(dimensions) eq 0 then dimensions = [289,289]

if not keyword_set(quiet) then message, 'Prepping '+trim(nimgs)+' image(s)...', /info

;---Main loop

for i=0L, nimgs-1 do begin

	;read FITS file
	if in_type eq 'default' then begin
		mreadfits, file[i], in, da, /quiet 
	;or use the iindex and idata inputs
	endif else begin
		in = iindex[i]
		da = idata[*,*,i]
	endelse
	
	;Skip ahead if the file was produced by MWA_PREP and doesn't need prepping
	want = where(strpos(in.history, 'MWA_PREP') ne -1, count)
	if count gt 0 then begin
		img = da
		hdr = in
		goto, already_prepped
	endif
	
	basename = file_basename(file[i])
	
	;Fix DATE_OBS tag for CASA files using filename and remove history tag
	if strmid(in.origin, 0, 4) eq 'CASA' then begin
		;remove CASA history, see header for details on why
		casa_history = in.history
		in = rem_tag(in, 'history')
		
		;remove linefeed character from ORIGIN because it interferes with readability by certain libraries
		pos = stregex(in.origin, string(10b))
		if pos ne -1 then in.origin = strmid(in.origin, 0, pos)
		
		;get time from the filename
		hr_min_sec = stregex(basename, 't([0-9]){6}', /extract)
		if hr_min_sec eq '' then begin
			message, 'Could not read time from filename for: '+basename+'. DATE_OBS may reflect OBSID start time.', /info
		endif else begin
			hr_min_sec = strjoin(strmid(hr_min_sec, indgen(3)*2 +1, 2), ':')
			yyyy_mm_dd = strmid(in.date_obs, 0, 10)
			date_obs = yyyy_mm_dd+'T'+hr_min_sec+'.000000'

			in.date_obs = date_obs
			in.date_d$obs = date_obs
		endelse	
	endif 	
	
	;convert FITS header into WCS structure
	wcs = fitshead2wcs(in)
	
	;image dimensions
	naxis = [in.naxis1, in.naxis2]
	
	;get sun-center position in right ascension and declination
	sun_radec = [0.,0.]
	!quiet=1
	convert_stereo_lonlat, in.date_obs, sun_radec, 'HPC', 'GEI'
	!quiet=0
	
	;convert ra/dec to degrees
	sun_radec = sun_radec*!radeg

	;sun-center position in pixels on the image
	crpix = (wcs_get_pixel(wcs, sun_radec))[0:1]
	
	if crpix[0] lt 0 OR crpix[0] gt naxis[0]-1 OR crpix[1] lt 0 OR crpix[1] gt naxis[1]-1 then begin
		message, 'Oops, the sun-center position is not on the image: '+basename, /info
		return
	endif
	
	;returns [solar P angle, solar B0 angle, solar radius in arcsec]
	sun_angles = pb0r(in.date_obs, /earth, l0=l0, /arcsec)

	;plate scale (arcsec per pixel)	
	cdelt = abs([in.cdelt1, in.cdelt2])*3600.
	
	;roll_angle = 360 - solar P angle
	roll_angle = 360.-sun_angles[0]

	;pc = coordinate transformation matrix. see fitshead2wcs lines 628-643 for details
	pc = dblarr(2,2)
	lambda = cdelt[1]/cdelt[0]
	pc[0,0] = cos(roll_angle/!radeg)
	pc[1,0] = sin(roll_angle/!radeg)/lambda
	pc[0,1] = -lambda*sin(roll_angle/!radeg)
	pc[1,1] = cos(roll_angle/!radeg)
	
	;create wcs coordinate structure. see header of fitshead2wcs for details
	wcs_hpc = { $
	   COORD_TYPE: 'Helioprojective-Cartesian', $ 
	   WCSNAME: 'Helioprojective-Cartesian', $
	   NAXIS: naxis, $ 
	   VARIATION: 'CROTA', $
	   COMPLIANT: 1B, $
	   PROJECTION: 'TAN', $ 
	   IX: 0L, $ ;
	   IY: 1L, $
	   CRPIX: crpix, $ 
	   CRVAL: dblarr(2), $
	   CTYPE: ['HPLN-TAN', 'HPLT-TAN'], $
	   CNAME: strarr(2), $
	   CUNIT: strarr(2)+'arcsec', $
	   CDELT: cdelt, $ 
	   ROLL_ANGLE: roll_angle, $ 
	   PC: pc, $ 
	   SIMPLE: 1b, $
	   TIME: wcs.time, $
	   POSITION: wcs.position }

	;re-grid the data if the keyword is enabled. 
	if keyword_set(regrid) then begin
		if keyword_set(debug) then begin
			;right ascension and declination in degrees
			radec = wcs_get_coord(wcs, /force_proj, /pos_long)
			;remove frequency and stokes dimensions
			radec = (reform(radec))[0:1,*,*] 	
			;ra_range = [0,360] & dec_range = [-90,90] ;deg
			;ra = radec[0,*,*] & dec = radec[1,*,*]

			;convert ra & dec to radians
			rad_radec = radec/!radeg

			;convert ra & dec (GEI) to helioprojective cartesian (HPC) coordinates
			hpc_coord = rad_radec
			!quiet=1
			convert_stereo_lonlat, in.date_obs, hpc_coord, 'GEI', 'HPC'
			!quiet=0
	
			;HPC Longitude and Latitude
			hpc_lon = reform(hpc_coord[0,*,*])
			hpc_lat = reform(hpc_coord[1,*,*])
	
			;X,Y coordinates using standard TAN projection. See Thompson 2005, Sec 4.1
			x = ( !radeg*tan(hpc_lon) )*3600.
			y = ( !radeg*(tan(hpc_lat)/cos(hpc_lon)) )*3600.

			;radial distance from sun-center in arcsec
			r = sqrt(x^2. + y^2.)	
			;radial distance from sun-center in rsun
			rsun = r/sun_angles[2] 
	
			limb = where(rsun le 1, count)
		endif
	
		;Retrieve the x and y arcsec coordinates of the new HPC grid
		new_xy = wcs_get_coord(wcs_hpc)
		new_x = reform(new_xy[0,*,*])
		new_y = reform(new_xy[1,*,*])
		
		if keyword_set(debug) then begin
			new_r = sqrt(new_x^2 + new_y^2)
			new_rsun = new_r/sun_angles[2]
		endif
		
		;Now determine the helioprojective longitudes and latidues. See Thompson 2005, Sec 4.1
		new_hpc_lon = atan(new_x/(3600.*!radeg))
		new_hpc_lat = atan( (cos(new_hpc_lon)*new_y) / (3600.*!radeg) )
		new_hpc_coord = transpose([[[new_hpc_lon]],[[new_hpc_lat]]], [2,0,1])
		
		;Convert these in the right ascension and declination (radians)
		new_rad_radec = new_hpc_coord
		!quiet=1
		convert_stereo_lonlat, in.date_obs, new_rad_radec, 'HPC', 'GEI', /pos_long
		!quiet=0
		
		;convert ra/dec to degrees
		new_radec = new_rad_radec*!radeg
		
		;Now determine the subpixel locations of the new grid points for the interpolation
		new_px = (wcs_get_pixel(wcs, new_radec))[0:1,*,*]
		
		;reform pixel coordinates in a 2 by #pixel array
		npx = product(long64(naxis))
		new_px_pairs = reform(new_px, [2,npx])
		
		;interpolate the data onto the new grid and then reform back to an image array
		interp_da = interpolate(da, cubic=-0.5, missing=0., new_px_pairs[0,*], new_px_pairs[1,*])
		interp_da = reform(interp_da, naxis)
		
		da = interp_da										   	
	endif
	
	if keyword_set(rotate) then begin
		da_rect = da
		wcs_rect = wcs_hpc
		;rotate the image using cubic interpolation
		wcs_rectify, da_rect, wcs_rect, cubic=-0.5
		
		;create solarsoft map structure
		wcs2map, da_rect, wcs_rect, map, id='MWA'
		
		;first clip the image to a bit larger than the final clip size, which speeds 
		;up the rescaling and ensures that the final dimensions are consistent. 
		if clip ne 0 then begin 
			range = [-960.,960]*(clip/2)
			sub_map, map, smap, xrange=range+[-100,100], yrange=range+[-100,100]
		endif else smap = map
		
		;scale image to prescribed plate scale
		if plate_scale ne 0 then begin
			gmap = grid_map(smap, dx=plate_scale, dy=plate_scale)
		endif else gmap = map
	
		;crop image to CLIP*r_sun
		if clip ne 0 then begin
			sub_map, gmap, map_final, xrange=range, yrange=range, dimensions=dimensions
			;ensure that the map is centered (i.e. xcen,ycen = 0,0)
			cen = [map_final.xc, map_final.yc]
			if cen[0] ne 0 OR cen[1] ne 0 then begin 
				shift_xy = -1.*cen
				map_final = shift_map(temporary(map_final), shift_xy[0], shift_xy[1])
			endif
			sz = size(map_final.data, /dimensions)
			if sz[0] ne sz[1] then message, 'ERROR, stopped... Image dimensions not consistent. Needs debugging.', /info
		endif else map_final = gmap
		
		if plate_scale + clip ne 0 then begin
			img = map_final.data 
			map2wcs, map_final, wcs_final
		endif else begin
			map_final = map
			wcs_final = wcs_rect
			img = da_rect
		endelse
	
		;roll beam angle accordingly 
		;NEED TO FIX THIS AND UPDATE ARCHIVE ACCORDINGLY. BPA EXPRESSION IN USE BELOW IS WRONG!!!
		;Expression commented out below works for 2015/09/21 obs, but seems to not work for some other cases, need to review
		;bpa = in.bpa-(360-wcs_hpc.roll_angle)
		bpa = wcs_hpc.roll_angle - in.bpa
		if bpa lt 0. then bpa = bpa + 360
				
	endif else begin
		img = da
		wcs_final = wcs_hpc
		
		bpa = in.bpa
	endelse
	
	;convert beam axes to arcsec 
	bmin = in.bmin*60.*60. 
	bmaj = in.bmaj*60.*60.
	
	;solar (optical) radius in pixels
	r_sun = sun_angles[2] / wcs_final.cdelt[0]	
	
	;generate a pixel mask for the disk
	if i eq 0 then begin 
		;arrays of only x- and y-values wrt SUN center
		yy = (indgen(wcs_final.naxis[1])-wcs_final.crpix[1]) ## (1. + fltarr(wcs_final.naxis[0])) 
		xx = (1. + fltarr(wcs_final.naxis[0])) ## (indgen(wcs_final.naxis[1])-wcs_final.crpix[0]) 
		rr = sqrt(xx^2 + yy^2) ;array of radial distaces from SUN center for each pixel
		rsol = rr / r_sun ;array of radial distances from center for each pixel in solar radii
		
		disk_mask = where(rsol lt 1.5, count, complement=off_mask)
		if count eq 0 then message, 'ERROR: failed to compute limb mask, needs debugging.'
	endif

	;mwa channel numbers
	channel = strjoin(trim(round(round(((in.crval3+[-0.5,0.5]*in.cdelt3)/1E6))/1.28)), '-')	

	;get OBSID from the filename
	obsid = strmid(basename, 0, strpos(basename, '_'))
	
	;create fits header from wcs structure
	hdr = wcs2fitshead(wcs_final, img, /add_xcen, /add_roll, oldhead=in)	

	;add history
	sxaddhist, progname+' '+progver+' run at: '+systim(), hdr
	
	;add in spectral and STOKES position tags because they were stripped by wcs2fitshead
	
	add_tags = ['SPECSYS', $
				'CUNIT4', 'CDELT4', 'CRVAL4', 'CRPIX4', 'CTYPE4', $
				'CUNIT3', 'CDELT3', 'CRVAL3', 'CRPIX3', 'CTYPE3', $
				'EXTEND', 'BSCALE', 'BZERO', 'BUNIT', 'EQUINOX']
	
	for j=0, n_elements(add_tags)-1 do begin
		if tag_exist(in, add_tags[j], index=tag_ind) then $
			sxaddpar, hdr, add_tags[j], in.(tag_ind), after='CUNIT2'
	endfor	
	
	;---Add more tags
	
	;rms around image edgeß
	rms = rms(img[edge_pixels(img, border=5)])
	
	;fraction of disk pixels over 3 sigma
	want = where(img[disk_mask] ge rms*3., count)
	dsk_3sig = float(count) / n_elements(disk_mask)
	
	cspeed = 299792458.0000
	
	add_struct = {DATAMIN:min(img), DATAMAX:max(img), DATAMEAN:mean(img), $
				  DATAMEDN:median(img), DATATOT:total(img), DATARMS:rms(img), $
				  RMS:rms, SNR:max(img)/rms, DSK_SNR:mean(img[disk_mask])/rms, $
				  DSK_RMS:rms(img[disk_mask]), DSK_FRAC:total(img[disk_mask]) / total(img), $
				  DSK_3SIG:dsk_3sig, TELESCOP:'MWA', CHANNEL:channel, OBSID:obsid, $
				  RESTFRQ:in.crval3, DELTAFRQ:in.cdelt3, FRQUNIT:'Hz', $
				  RESTWAVE:cspeed/in.crval3, WAVELNTH:cspeed/in.crval3, $
				  WAVEUNIT:'m', R_SUN:r_sun, LVL_NUM:1, BPA:bpa, BMIN:bmin, $
				  BMAJ:bmaj}
	
	add_tags = tag_names(add_struct)
	for j=0, n_elements(add_tags)-1 do sxaddpar, hdr, add_tags[j], add_struct.(j), after='CUNIT2'	
		
	;convert header to structure
	hdr = fitshead2struct(hdr)
	
	tags = tag_names(hdr)
	in_tags = tag_names(in)

	;jump point for files that have already been through 
	already_prepped:

	;populate final output variables
	if not keyword_set(no_output) then begin 
		if i eq 0 then begin 
			data = img 
			index = hdr
			out_dim = size(data, /dimensions)
		endif else begin
			;ensure that the re-sized image actually matches the previous loops
			if total(size(img, /dimensions) ne out_dim) gt 0 then begin
				;if the image sizes aren't equal and the defaults are set, then there's an error. 
				if keyword_set(plate_scale) AND keyword_set(clip) then begin 
					message, 'Uh oh, output image dimensions not matching. Needs debugging. You can run MWA_PREP separately on each file in the meantime.', /info
					stop
					return
				;If rotation, clipping, and scaling have been disabled, then it's on the user.
				endif else begin
					message, 'Oops, input images have different sizes. You can enable the ROTATE, CLIP, and PLATE_SCALE keywords (the default) or run MWA_PREP separately on each file.', /info
					return 
				endelse
			endif
			
			data = [[[data]], [[img]]]
			index = concat_struct(index, hdr)
		endelse 	
	endif
	
	;---Establish write directory and create it if necessary

	if keyword_set(write) then begin
		if size(write, /tname) eq 'STRING' then odir = write else odir = file_dirname(file[i])+'/'
		if file_test(odir, /dir) eq 0 then begin
			file_mkdir, odir
			if not keyword_set(quiet) then message, 'Created write dir: '+odir, /info
		endif
		;Write file after checking that it's not already prepped.
		if strpos(basename, '_prep') eq -1 then begin 
			fname = str_replace(str_replace(basename, '.fits', '_prep.fits'), '.FITS', '_prep.FITS')
			mwritefits, hdr, img, outfile=odir+'/'+fname
		endif		
	endif	
	
	if keyword_set(lstop) then stop
	
	if not keyword_set(quiet) and n_elements(file) gt 5 then progress_bar, i, n_elements(file)-1
		
endfor

if keyword_set(runtime) then message, 'Run time (s): '+trim(systime(/seconds)-t1), /info

return
end

